#!/usr/bin/perl -w
#
# registerReceiver.pl
#
# Copyright (C) 2011-2013 Cesnet z.s.p.o
#
# Use of this source is governed by a BSD-style license, see LICENSE file.

use strict;
use Getopt::Std;
use Switch;
use DBI;
use DBD::mysql;
use DateTime;
use Sys::Syslog qw(:DEFAULT setlogsock);
Sys::Syslog::setlogsock('unix');
use Carp;
use FindBin qw($RealBin $RealScript);
use lib "$FindBin::RealBin/../lib";
use WardenCommon;



################################################################################
#				VARIABLES
################################################################################
our $VERSION = "2.2";
my $etc = "$FindBin::RealBin/../etc";



################################################################################
#                               FUNCTIONS
################################################################################
sub usage {
  print "Usage: $RealScript [-h -o -n <hostname> -r <requestor> -t <type> -i <ip_net_client>]\n";
  exit 1;
}


sub help {
  print "$RealScript [-h -o -n <hostname> -r <requestor> -t <type> -i <ip_net_client>]\n";
  print "-h     print this text and exit\n";
  print "-n     hostname of receiver\n";
  print "-r     client registration requestor\n";
  print "-t     type of received events or '_any_' for receiving of all types of events\n";
  print "-o     enable receiving of own events\n";
  print "-i     CIDR of receiver\n";
  exit 0;
}



################################################################################
#                               MAIN
################################################################################
our ($opt_n, $opt_r, $opt_t, $opt_o, $opt_i, $opt_h);

if ($#ARGV == -1) {usage}

die usage unless getopts("n:r:t:i:ho");
my $hostname            = $opt_n;
my $requestor           = $opt_r;
my $type		= $opt_t;
my $ip_net_client       = $opt_i;
my $help                = $opt_h;
my $receive_own_events	= "f";
if ($opt_o) {$receive_own_events = "t"}

# catch help param
if ($help) {help}

# check validity of IP address (CIDR format)
if ($ip_net_client !~ /^(([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])(\/(\d|[1-2]\d|3[0-2]))$/) {
  WardenCommon::errMsg("Enter correct IP address in CIDR format!");
}

# superuser controle
my $UID = $<;
if ($UID != 0) {WardenCommon::errMsg("You must be root for running this script!")}

# check parameters definition
switch () {
  case {!defined $hostname} {WardenCommon::errMsg("Parameter 'hostname' is not defined!")}
  case {!defined $requestor} {WardenCommon::errMsg("Parameter 'requestor' is not defined!")}
  case {!defined $type} {WardenCommon::errMsg("Parameter 'type' is not defined!")}
  case {!defined $receive_own_events} {WardenCommon::errMsg("Parameter 'receive_own_events' is not defined!")}
  case {!defined $ip_net_client} {WardenCommon::errMsg("Parameter 'ip_net_client' is not defined!")}
}

# defined variables by script
my $client_type		= "r";
my $valid		= "t";
my $registered		= DateTime->now;
my $service		= undef;
my $description_tags	= undef;

# read config file
my $conf_file = "$etc/warden-server.conf";
our $SYSLOG             = undef;
our $SYSLOG_VERBOSE     = undef;
our $SYSLOG_FACILITY	= undef;
our $DB_NAME            = undef;
our $DB_USER            = undef;
our $DB_PASS            = undef;
our $DB_HOST            = undef;
unless (do $conf_file) {
  die("Errors in config file '$conf_file': $@") if $@;
  die("Can't read config file '$conf_file': $!") unless defined $_;
  # if $_ defined, it's retvalue of last statement of conf, for which we don't care
}

# connect to DB
our $DBH = DBI->connect("DBI:mysql:database=$DB_NAME;host=$DB_HOST", $DB_USER, $DB_PASS, {RaiseError => 1, mysql_auto_reconnect => 1})
           || die "Could not connect to database: $DBH->errstr";

# check if receiver has been already registered or register it
my $sth = $DBH->prepare("SELECT registered FROM clients WHERE hostname = ? AND client_type = ? AND type = ? AND ip_net_client = ? LIMIT 1;") or die "Cannot prepare statement: " . $DBH->errstr;
$sth->execute($hostname, $client_type, $type, $ip_net_client) or die "Cannot execute statement: " . $sth->errstr;
my $result = $sth->fetchrow();
my $warden_server = WardenCommon::trim(`hostname -f`);
if (defined $result) {
  WardenCommon::sendMsg($SYSLOG, $SYSLOG_VERBOSE, $SYSLOG_FACILITY, "err", "Attempt to re-register the receiver [hostname: '$hostname', type: '$type', cidr: '$ip_net_client']");
  WardenCommon::errMsg("Receiver has already been registered at $warden_server in '$result'");
} else {
  $sth = $DBH->prepare("INSERT INTO clients VALUES (?,?,?,?,?,?,?,?,?,?,?);") or die "Cannot prepare statement: " . $DBH->errstr;
  my $ret_val = $sth->execute(undef, $hostname, $registered, $requestor, $service, $client_type, $type, $receive_own_events, $description_tags, $ip_net_client, $valid) or die "Cannot execute statement: " . $sth->errstr;
  if ($ret_val == 1) {
    WardenCommon::sendMsg($SYSLOG, $SYSLOG_VERBOSE, $SYSLOG_FACILITY, "info", "New receiver [hostname: '$hostname', type: '$type', cidr: '$ip_net_client'] was registered");
    WardenCommon::succMsg("Registration of receiver from $hostname was SUCCESSFUL!!!");
  } else {
    WardenCommon::errMsg("Registration of receiver from $hostname FAILED!!!");
  }
}
